﻿using System;
using System.Windows.Shapes;  // Rectangle을 사용하기 위해 필요
using System.Windows.Controls;  // UI 컨트롤 (DataGrid 등) 사용을 위해 필요
using System.Windows.Media;  // Color 및 SolidColorBrush 사용을 위해 필요
using System.Threading.Tasks;  // Task.Delay() 사용을 위해 필요
using System.IO;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.ComponentModel;
using ServiceStack.Redis;
using Newtonsoft.Json;
using System.Collections.ObjectModel;

namespace RedisWPFApp
{
    public partial class MainWindow : Window
    {
        // Redis connection setup (assuming _receiveRedis is defined elsewhere)
        private RedisClient _receiveRedis;

        // ObservableCollection to store the data and automatically update the UI
        private ObservableCollection<RealEstateData> allRealEstateData = new ObservableCollection<RealEstateData>();

        // BackgroundWorker to handle Redis data fetching
        private BackgroundWorker _recvWorker;

        public MainWindow()
        {
            InitializeComponent();

            // Initialize Redis connection
            _receiveRedis = new RedisClient("localhost", 6379); // Adjust Redis connection details

            // Initialize the BackgroundWorker
            _recvWorker = new BackgroundWorker();
            _recvWorker.DoWork += new DoWorkEventHandler(recvWorker_DoWork);
            _recvWorker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(recvWorker_RunWorkerCompleted);

            // Start receiving data
            _recvWorker.RunWorkerAsync();
        }

#if(false)
	private void recvWorker_DoWork(object sender, DoWorkEventArgs e)
	{
	    try
	    {
		// Redis에서 데이터를 꺼낼 때, BRPop을 사용하여 10초 대기
		var data = _receiveRedis.BRPop("QCK:11270016379155155155155", 10);

		if (data != null && data.Length > 0)
		{
		    string listName = Encoding.UTF8.GetString(data[0]);
		    string value = Encoding.UTF8.GetString(data[1]);

		    // Deserialize the JSON data to a RealEstateData object
                    DynRcvInterface dynInterface = JsonConvert.DeserializeObject<DynRcvInterface>(value);
		    RealEstateData realEstateData = JsonConvert.DeserializeObject<RealEstateData>(dynInterface.data);

		    // Deserialize the JSON data to a RealEstateData object
		    //var realEstateData = JsonConvert.DeserializeObject<RealEstateData>(value);

		    // Store the data to be used in RunWorkerCompleted
		    e.Result = realEstateData;
		}
	    }
	    catch (Exception ex)
	    {
		// Handle errors gracefully and store them in Result to pass to RunWorkerCompleted
		e.Result = ex.Message;
	    }
	}

	private void recvWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
	{
	    if (e.Error != null)
	    {
		// Handle errors if any occurred during DoWork
		MessageBox.Show("Error: " + e.Error.Message);
	    }
	    else if (e.Result != null)
	    {
		// If data was fetched successfully, update the UI
		if (e.Result is string error)
		{
		    // If there was an error message
		    MessageBox.Show("Error: " + error);
		}
		else
		{
		    // Cast e.Result to RealEstateData since it was serialized to a RealEstateData object
		    RealEstateData realEstateData = e.Result as RealEstateData;

		    if (realEstateData != null)
		    {
			// Add the new data to the ObservableCollection
			allRealEstateData.Add(realEstateData);
			SaveRealEstateDataToFile(realEstateData);			
		    }
		}
	    }
	    myDataGrid.ItemsSource = allRealEstateData;

	    // Optionally, restart the background worker to continue receiving data
	    _recvWorker.RunWorkerAsync();
	}
#endif
	private void recvWorker_DoWork(object sender, DoWorkEventArgs e)
	{
	    try
	    {
		// 회색 → 파란색으로 변경
		Dispatcher.Invoke(() => BlinkPanel(grayPanel, Colors.Gray, Colors.Blue));

		// Redis에서 데이터를 꺼낼 때, BRPop을 사용하여 10초 대기
		var data = _receiveRedis.BRPop("QCK:11270016379155155155155", 10);

		if (data != null && data.Length > 0)
		{
		    string listName = Encoding.UTF8.GetString(data[0]);
		    string value = Encoding.UTF8.GetString(data[1]);

		    // Deserialize the JSON data to a RealEstateData object
		    DynRcvInterface dynInterface = JsonConvert.DeserializeObject<DynRcvInterface>(value);
		    RealEstateData realEstateData = JsonConvert.DeserializeObject<RealEstateData>(dynInterface.data);

		    // Store the data to be used in RunWorkerCompleted
		    e.Result = realEstateData;
		}
	    }
	    catch (Exception ex)
	    {
		e.Result = ex.Message;
	    }
	}

	private void recvWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
	{
	    if (e.Error != null)
	    {
		MessageBox.Show("Error: " + e.Error.Message);
	    }
	    else if (e.Result != null)
	    {
		if (e.Result is string error)
		{
		    MessageBox.Show("Error: " + error);
		}
		else
		{
		    RealEstateData realEstateData = e.Result as RealEstateData;
		    if (realEstateData != null)
		    {
			allRealEstateData.Add(realEstateData);
			//SaveRealEstateDataToFile(realEstateData);

		        // UI 스레드에서 DataGrid 업데이트
			Dispatcher.Invoke(() => 
			{
				myDataGrid.ItemsSource = null;  // 바인딩 갱신을 위해 초기화
				myDataGrid.ItemsSource = allRealEstateData;
			    
				// 회색 → 빨간색으로 변경
				BlinkPanel(redPanel, Colors.Gray, Colors.Red);
			});
		    }
		}
	    }

	    // Optionally, restart the background worker to continue receiving data
	    _recvWorker.RunWorkerAsync();
	}

	// 깜빡이는 효과를 주는 메서드
	private async void BlinkPanel(Rectangle panel, Color fromColor, Color toColor)
	{
	    panel.Fill = new SolidColorBrush(toColor);
	    await Task.Delay(500);
	    panel.Fill = new SolidColorBrush(fromColor);
	}



	// Method to save realEstateData to a JSON file
	private void SaveRealEstateDataToFile(RealEstateData realEstateData)
	{
	    try
	    {
		// Serialize the RealEstateData object to JSON
		string json = JsonConvert.SerializeObject(realEstateData, Formatting.Indented);

		// Define the file path where you want to save the data
		//string filePath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments), "RealEstateData.json");
		//C:\Users\xterm\Downloads\NMM12
		string filePath = @"C:\Users\B210145_BK\Downloads\EstateDotNet\WinFoundFrm38\RealEstateData.json";

		// Write the JSON data to the file
		File.AppendAllText(filePath, json + Environment.NewLine); // Append to the file if it already exists

		// Notify the user
		//MessageBox.Show($"Data saved to file: {filePath}");
		//Console.WriteLine(json);
	    }
	    catch (Exception ex)
	    {
		// Handle any errors that occur during file writing
		MessageBox.Show($"Error saving data to file: {ex.Message}");
	    }
	}
    }
}

